@@ -58,16 +58,15 @@ sub test_leaks {
     my ($self) = @_;
     no_leaks_ok { $self->implementation->new } 'no leaks constructing dispatcher';
 
+    my $dispatcher = $self->implementation->new;
     no_leaks_ok {
-        my $dispatcher = $self->implementation->new;
-
         my $cv = AnyEvent->condvar;
         $dispatcher->create_watcher("/" => sub{ $cv->send }, type => "test");
         $dispatcher->trigger_event(path => "/", type => "test");
         timeout 1, sub { $cv->recv };
 
         $cv = AnyEvent->condvar;
-        $watch = $dispatcher->create_watcher("/second" => sub{ $cv->send }, type => "second-test");
+        $dispatcher->create_watcher("/second" => sub{ $cv->send }, type => "second-test");
         $dispatcher->trigger_event(path => "/second", type => "second-test");
         timeout 1, sub { $cv->recv };
     } 'no leaks sending events through dispatcher';
@@ -111,4 +110,40 @@ sub test_ignore_session_events {
     is_deeply $rv, $event, "dispatcher called watcher with watcher event";
 }
 
+sub test_duplicate_watchers {
+    my ($self) = @_;
+    my $dispatcher = $self->implementation->new;
+
+    my $cv1 = AnyEvent->condvar;
+    $dispatcher->create_watcher('/' => sub{ $cv1->send(shift) }, type => "test");
+    my $cv2 = AnyEvent->condvar;
+    $dispatcher->create_watcher("/" => sub{ $cv2->send(shift) }, type => "test");
+
+    my $event = {type => 1, state => 2, path => 'test-path'};
+    $dispatcher->trigger_event(path => "/", type => "test", event => $event);
+
+    my $rv; timeout 1, sub { $rv = $cv1->recv };
+    is_deeply $rv, $event, "dispatcher called first watcher with event";
+
+    $rv = undef; timeout 1, sub { $rv = $cv2->recv };
+    is_deeply $rv, $event, "dispatcher called duplicate watcher with event";
+}
+
+sub test_duplicate_watchers_leaks {
+    my ($self) = @_;
+    my $dispatcher = $self->implementation->new;
+
+    no_leaks_ok {
+        my $cv1 = AnyEvent->condvar;
+        $dispatcher->create_watcher("/" => sub{ $cv1->send }, type => "test");
+
+        my $cv2 = AnyEvent->condvar;
+        $dispatcher->create_watcher("/" => sub{ $cv2->send }, type => "test");
+
+        $dispatcher->trigger_event(path => "/", type => "test");
+        timeout 1, sub { $cv1->recv };
+        timeout 1, sub { $cv2->recv };
+    } 'no leaks with duplicate watchers';
+}
+
 1;
